# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-23 23:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

import swapper


class Migration(migrations.Migration):

    dependencies = [
        ('cities', '0004_rename_languages_to_language_codes'),
        swapper.dependency('cities', 'City'),
    ]

    operations = [
        migrations.AddField(
            model_name='postalcode',
            name='city',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='postal_codes', to=swapper.get_model_name('cities', 'City')),
        ),
        migrations.AddField(
            model_name='postalcode',
            name='district',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='postal_codes', to='cities.District'),
        ),
        migrations.AddField(
            model_name='postalcode',
            name='region',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='postal_codes', to='cities.Region'),
        ),
        migrations.AddField(
            model_name='postalcode',
            name='subregion',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='postal_codes', to='cities.Subregion'),
        ),
    ]
