# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-23 23:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

import swapper


class Migration(migrations.Migration):

    dependencies = [
        ('cities', '0002_continent_models_and_foreign_keys'),
        swapper.dependency('cities', 'Continent'),
        swapper.dependency('cities', 'Country'),
        swapper.dependency('cities', 'City'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cities', to=swapper.get_model_name('cities', 'Country')),
        ),
        migrations.AlterField(
            model_name='city',
            name='region',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='cities.Region'),
        ),
        migrations.AlterField(
            model_name='city',
            name='subregion',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='cities.Subregion'),
        ),
        migrations.AlterField(
            model_name='continent',
            name='alt_names',
            field=models.ManyToManyField(to='cities.AlternativeName'),
        ),
        migrations.AlterField(
            model_name='continent',
            name='slug',
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name='country',
            name='tld',
            field=models.CharField(max_length=5, verbose_name='TLD'),
        ),
        migrations.AlterField(
            model_name='district',
            name='city',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='districts', to=swapper.get_model_name('cities', 'City')),
        ),
        migrations.AlterField(
            model_name='region',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='regions', to=swapper.get_model_name('cities', 'Country')),
        ),
        migrations.AlterField(
            model_name='subregion',
            name='region',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subregions', to='cities.Region'),
        ),
    ]
